@@ -586,8 +586,8 @@
                       method.lik = method.lik, trend = trend,
                       loglik = loglik.max,
                       npars = npars,
-                      AIC = (loglik.max - npars),
-                      BIC = (loglik.max - 0.5 * log(n) * npars),
+                      AIC = -2 * (loglik.max - npars),
+                      BIC = -2 * (loglik.max - 0.5 * log(n) * npars),
                       parameters.summary = par.su,
                       info.minimisation.function = lik.minim,
                       max.dist = max.dist,
